<?php

use Illuminate\Translation\ArrayLoader;
use Illuminate\Translation\Translator;
use Myriad\Validators\MyriadValidators;
use PHPUnit\Framework\TestCase;

class ValidatorTestCase extends TestCase
{

    public function testValidateCsv()
    {
        $v = $this->createValidator(['foo' => 'a,b,c'], ['foo' => 'csv']);
        $this->assertTrue($v->passes(), 'test that simple csv passes');

        $v = $this->createValidator(['foo' => 'a,,b'], ['foo' => 'csv:required']);
        $this->assertFalse($v->passes(), 'test that required sub-rule fails when csv item is empty');

        $v = $this->createValidator(['foo' => 'a,b,c'], ['foo' => 'csv:required']);
        $this->assertTrue($v->passes(), 'test that required sub-rule passes when all csv items are non-empty');

        $v = $this->createValidator(['foo' => 'a,b,c'], ['foo' => 'csv:min:2']);
        $this->assertFalse($v->passes(), 'test that sub-rules with extra parameters work as expected 1');

        $v = $this->createValidator(['foo' => 'aa,bb,cc'], ['foo' => 'csv:min:2']);
        $this->assertTrue($v->passes(), 'test that sub-rules with extra parameters work as expected 2');

        $v = $this->createValidator(['foo' => 'aa,bb,cc'], ['foo' => 'csv:min:2\in:aa,bb']);
        $this->assertFalse($v->passes(), 'test that multiple sub-rules work as expected 1');

        $v = $this->createValidator(['foo' => 'aa,bb,cc'], ['foo' => 'csv:min:2\in:aa,bb,cc']);
        $this->assertTrue($v->passes(), 'test that multiple sub-rules work as expected 2');
    }

    public function testValidateUuid()
    {
        $v = $this->createValidator(['foo' => '0b7bff99-3929-4ba3-887e-772bd21f38c0'], ['foo' => 'uuid-version']);
        $this->assertTrue($v->passes(), 'test that a v4 uuid with dashes passes');

        $v = $this->createValidator(['foo' => '0b7bff9939294ba3887e772bd21f38c0'], ['foo' => 'uuid-version']);
        $this->assertTrue($v->passes(), 'test that a v4 uuid without dashes passes');

        $v = $this->createValidator(['foo' => '0b7bff9939294ba3887e772bd21f38c'], ['foo' => 'uuid-version']);
        $this->assertFalse($v->passes(), 'test that an invalid uuid does not pass');

        $v = $this->createValidator(['foo' => 'fd8b5544-f6bd-11e5-9ce9-5e5517507c66'], ['foo' => 'uuid-version']);
        $this->assertTrue($v->passes(), 'test that a v1 uuid passes');

        // test ensure v1 uuid
        $v = $this->createValidator(['foo' => 'fd8b5544-f6bd-11e5-9ce9-5e5517507c66'], ['foo' => 'uuid-version:1']);
        $this->assertTrue($v->passes(), 'test that a v1 uuid passes when the explicit version parameter is 1');

        // test ensure v4 uuid fails with v1 uuid
        $v = $this->createValidator(['foo' => 'fd8b5544-f6bd-11e5-9ce9-5e5517507c66'], ['foo' => 'uuid-version:4']);
        $this->assertFalse($v->passes(), 'test that a v1 uuid fails when the explicit version parameter is 4');
    }

    public function testValidateLocale()
    {
        $v = $this->createValidator(['foo' => 'en-us'], ['foo' => 'locale']);
        $this->assertTrue($v->passes(), 'test that a valid language tag passes');

        $v = $this->createValidator(['foo' => 'eN-uS'], ['foo' => 'locale']);
        $this->assertTrue($v->passes(), 'test that case doesn\'t matter');

        $v = $this->createValidator(['foo' => 'e-us'], ['foo' => 'locale']);
        $this->assertFalse($v->passes(), 'test that the language component fails with 1 character');

        $v = $this->createValidator(['foo' => 'enn-us'], ['foo' => 'locale']);
        $this->assertTrue($v->passes(), 'test that the language component accepts 3 characters');

        $v = $this->createValidator(['foo' => 'ennn-us'], ['foo' => 'locale']);
        $this->assertFalse($v->passes(), 'test that the language component fails with 4 characters');

        $v = $this->createValidator(['foo' => '11-us'], ['foo' => 'locale']);
        $this->assertFalse($v->passes(), 'test that the language component fails with non-alpha characters');

        $v = $this->createValidator(['foo' => 'en-s'], ['foo' => 'locale']);
        $this->assertFalse($v->passes(), 'test that the region component fails with 1 alpha character');

        $v = $this->createValidator(['foo' => 'en-sss'], ['foo' => 'locale']);
        $this->assertFalse($v->passes(), 'test that the region component fails with 3 alpha character');

        $v = $this->createValidator(['foo' => 'en-ssss'], ['foo' => 'locale']);
        $this->assertFalse($v->passes(), 'test that the region component fails with 4 alpha character');

        $v = $this->createValidator(['foo' => 'en-12'], ['foo' => 'locale']);
        $this->assertFalse($v->passes(), 'test that the region component fails with 2 numeric characters');

        $v = $this->createValidator(['foo' => 'en-123'], ['foo' => 'locale']);
        $this->assertTrue($v->passes(), 'test that the region component passes with 3 numeric characters');

        $v = $this->createValidator(['foo' => 'en-1234'], ['foo' => 'locale']);
        $this->assertFalse($v->passes(), 'test that the region component fails with 4 numeric characters');

        $v = $this->createValidator(['foo' => 'ab-123-abc-def-1234-asd123'], ['foo' => 'locale']);
        $this->assertTrue($v->passes(), 'test that any number of extensions are allowed');
    }

    private function createValidator($data, $rules)
    {
        return new MyriadValidators($this->getRealTranslator(), $data, $rules);
    }

    /**
     * @return Translator
     */
    private function getRealTranslator(): Translator
    {
        return new Translator(new ArrayLoader, 'en');
    }

}
